#include <cmath>
#include <cstdlib>
#include <windows.h>
#include <gl/gl.h>

using namespace std;

//================================================
// logical function that should be customized
//================================================
//		F(x,y)
// Returns true for points that should be coloured
// and false for other ones.
//================================================
//
bool F(float x, float y)
{
    //*
	return abs(y)>x*x && y<4.0f
		|| abs(x)/2+abs(y-4.0f)<1.0f
		|| x*x+(y-6.5f)*(y-6.5f)<2.25f
        || y>-2	&& abs(x)>2	&& (abs(x)-2)/2+(y-4)/6<0
        || y>8 && abs(x)<1 && (y-abs(x)*2<8 || y+abs(x)*2<10)
		;
	/*/
	/*
	float z = x*x + y*y;
	return z<4
	   || z> 9 && z<16
	   || z>25 && z<36
	   || z>49 && z<64
	   || z>81 && z<100
	   ;
	   */
}


//================================================
// forward declaration of functions
//================================================
//
LRESULT CALLBACK WndProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
void EnableOpenGL ();
void DisableOpenGL();


//================================================
//  handles - global vars
//================================================
//
WNDCLASS wc;
HWND     hWnd = NULL;
HDC      hDC = NULL;
HGLRC    hRC = NULL;


//================================================
//		Paint()
// Iterates through all points of the window and calls F(x,y) for every point.
// It draws each sequence of coloured points by a single line.
//================================================
//
float Range = 10.0f;    // x and y change within Range

void Paint()
{
	float w=512.0f,
		h=512.0f;

	float dx = fmin(1.0f/w, 1.0f/h);
	float dy = dx;

    glClearColor (0.0f, 0.0f, 0.0f, 0.0f);
    glClear(GL_COLOR_BUFFER_BIT);

	glBegin(GL_LINES);

		// draw coloured areas
	glColor3f (1.0f, 1.0f, 0.0f);
	
	for(float y=-1.0f; y<=1.0f+dy; y+=dy) {
		float x0=-1.0f;
		bool f=false;
		
		for(float x=-1.0f; x<=1.0f+dx; x+=dx) {
			bool f2;
			if (x>=1.0f)  f2=false;
			else          f2=F(x*Range, y*Range);
			
			if (!f && f2)  x0 = x;
			else
			if (f && !f2) {
				glVertex2f(x0, y);
				glVertex2f( x, y);
			}

			f = f2;
		}
	}
	
		// draw auxiliary markup lines
		//
	glColor4f(0.2f, 0.2f, 0.2f, 0.5f);

	for(float y=-1.0f; y<=1.0f; y+=1.0f/Range){
		glVertex2f(-1.0f, y);
		glVertex2f( 1.0f, y);
	}
	
	for(float x=-1.0f; x<=1.0f; x+=1.0f/Range){
		glVertex2f(x, -1.0f);
		glVertex2f(x,  1.0f);
    }

		// draw OX and OY axes
		//
	glColor3f (0.5f, 1.0f, 1.0f);

	glVertex2f(-1.0f, 0.0f);
	glVertex2f( 1.0f, 0.0f);
	
	glVertex2f( 0.0f, -1.0f);
	glVertex2f( 0.0f,  1.0f);

        // finish drawing
    glEnd ();
    
        // update screen
    SwapBuffers (hDC);
}


//================================================
//		MAIN function
//================================================
//
int WINAPI WinMain (HINSTANCE hInstance,
                    HINSTANCE hPrevInstance,
                    LPSTR lpCmdLine,
                    int iCmdShow)
{
//    WNDCLASS wc;
//    HWND hWnd;
//    HDC hDC;
//    HGLRC hRC;        
    MSG msg;
    BOOL bQuit = FALSE;
    float theta = 0.0f;

    	// register window class
    wc.style = CS_OWNDC;
    wc.lpfnWndProc = WndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor (NULL, IDC_HAND);		// IDC_ARROW
    wc.hbrBackground = (HBRUSH) GetStockObject (BLACK_BRUSH);
    wc.lpszMenuName = NULL;
    wc.lpszClassName = "GLSample";
    RegisterClass(&wc);

    	// create main window
    hWnd = CreateWindow (
		"GLSample", "OpenGL Sample", 
		WS_CAPTION | WS_POPUPWINDOW | WS_VISIBLE 
            | WS_BORDER,
		0, 0, 512, 512,
		NULL, NULL, hInstance, NULL);

    	// init OpenGL
    EnableOpenGL();

    Range = 10.0f;
	Paint();

    	// program's main loop for event processing
    while (!bQuit) {
            // read messages from the queue
        if (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE)) {
                // handle or dispatch messages
            if (msg.message == WM_QUIT) {
                bQuit = TRUE;
            }
            else {
                TranslateMessage(&msg);
                DispatchMessage (&msg);
            }
        }
        else {
            // idle
            // ...
        }
    }

    	// done with OpenGL
    DisableOpenGL();

    	// destroy window
    DestroyWindow(hWnd);

    return msg.wParam;
}


//================================================
// window functions
//================================================

LRESULT CALLBACK WndProc (HWND hWnd, UINT message,
                          WPARAM wParam, LPARAM lParam)
{

    switch (message)
    {
    case WM_CREATE:
        return 0;
    case WM_CLOSE:
        PostQuitMessage (0);
        return 0;

    case WM_DESTROY:
        return 0;

    case WM_KEYDOWN:
        switch (wParam)
        {
        case VK_ESCAPE:			// exit app if ESCAPE has been pressed
            PostQuitMessage(0);
            return 0;
        case VK_F5:
        	Paint();
        	return 0;
        case VK_UP:
            Range += 1.0f;
            Paint();
        	return 0;
        case VK_DOWN:
            if (Range>=2) Range -= 1.0f;
            Paint();
        	return 0;
        }
        return 0;

    default:
        return DefWindowProc (hWnd, message, wParam, lParam);
    }
}


//================================================
// Init OpenGL with standard params
//================================================

void EnableOpenGL()
{
    PIXELFORMATDESCRIPTOR pfd;
    int iFormat;

	    // get the Device Context
    hDC = GetDC (hWnd);

	    // set the Pixel Format
    ZeroMemory (&pfd, sizeof (pfd));
    pfd.nSize = sizeof (pfd);
    pfd.nVersion = 1;
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
    pfd.iPixelType = PFD_TYPE_RGBA;
    pfd.cColorBits = 24;
    pfd.cDepthBits = 16;
    pfd.iLayerType = PFD_MAIN_PLANE;
    iFormat = ChoosePixelFormat (hDC, &pfd);
    SetPixelFormat (hDC, iFormat, &pfd);

		// init render context
    hRC = wglCreateContext( hDC );
    wglMakeCurrent( hDC, hRC );
}


//================================================
// Done OpenGL
//================================================

void DisableOpenGL()
{
		// disconnect OpenGL
    wglMakeCurrent (NULL, NULL);
    
    	// release resources
    wglDeleteContext (hRC);
    ReleaseDC(hWnd, hDC);
}
